@@ -36,6 +36,7 @@
 #include <scdirlist.h>
 
 #include <new>
+#include <stdexcept>
 #include <string.h>
 #include <unistd.h>
 #include <sys/stat.h>
@@ -83,6 +84,12 @@
 #include <chemistry/qc/scf/linkage.h>
 #include <chemistry/qc/dft/linkage.h>
 #include <chemistry/qc/mbpt/linkage.h>
+#ifdef HAVE_SC_SRC_LIB_CHEMISTRY_QC_MBPTR12
+#  include <chemistry/qc/mbptr12/linkage.h>
+#endif
+#ifdef HAVE_SC_SRC_LIB_CHEMISTRY_QC_CINTS
+#  include <chemistry/qc/cints/linkage.h>
+#endif
 //#include <chemistry/qc/psi/linkage.h>
 #include <util/state/linkage.h>
 #ifdef HAVE_SC_SRC_LIB_CHEMISTRY_QC_CC
@@ -94,9 +101,11 @@
 
 #ifdef HAVE_MPI
 #include <mpi.h>
+#include <util/group/messmpi.h>
 #endif
 
 using namespace std;
+using namespace sc;
 
 #include "mpqcin.h"
 
@@ -128,19 +137,10 @@ clean_up(void)
   MessageGrp::set_default_messagegrp(0);
   ThreadGrp::set_default_threadgrp(0);
   SCMatrixKit::set_default_matrixkit(0);
+  Integral::set_default_integral(0);
   RegionTimer::set_default_regiontimer(0);
 }
 
-static void
-out_of_memory()
-{
-  ExEnv::out() << "ERROR: mpqc: out of memory" << endl;
-  ExEnv::out().flush();
-  cout.flush();
-  cerr.flush();
-  abort();
-}
-
 #include <signal.h>
 #if defined(HAVE_MACHINE_FPU_H)
 #include <machine/fpu.h>
@@ -155,32 +155,33 @@ extern "C" {
 };
 void sigfpe_handler(int)
 {
-  long fp_control = ieee_get_fp_control();
+  unsigned long fp_control = ieee_get_fp_control();
   int fatal = 0;
   if (fp_control & IEEE_STATUS_INV) {
-      ExEnv::out() << "SGIFPE: invalid operation" << endl;;
+      ExEnv::outn() << "SIGFPE: invalid operation" << endl;
       fatal = 1;
     }
   if (fp_control & IEEE_STATUS_DZE) {
-      ExEnv::out() << "SGIFPE: divide by zero" << endl;;
+      ExEnv::outn() << "SIGFPE: divide by zero" << endl;
       fatal = 1;
     }
   if (fp_control & IEEE_STATUS_OVF) {
-      ExEnv::out() << "SGIFPE: overflow" << endl;;
+      ExEnv::outn() << "SIGFPE: overflow" << endl;
       fatal = 1;
     }
   if (fp_control & IEEE_STATUS_UNF) {
-      //ExEnv::out() << "SGIFPE: underflow" << endl;;
+      //ExEnv::outn() << "SIGFPE: underflow" << endl;
     }
   if (fp_control & IEEE_STATUS_INE) {
-      //ExEnv::out() << "SGIFPE: inexact" << endl;;
+      //ExEnv::outn() << "SIGFPE: inexact" << endl;
     }
+  // throw'ing due to an FPE will violate nothrow guarantees, so abort
   if (fatal) abort();
 }
 #endif
 
 int
-main(int argc, char *argv[])
+try_main(int argc, char *argv[])
 {
   //trash_stack();
 
@@ -188,7 +189,6 @@ main(int argc, char *argv[])
   int i;
   const char *devnull = "/dev/null";
   atexit(clean_up);
-  std::set_new_handler(out_of_memory);
 
 #if defined(__i386__) && defined(__GNUC__)
   // make floating point errors cause an exception (except for denormalized
@@ -210,14 +210,9 @@ main(int argc, char *argv[])
 
   ExEnv::init(argc, argv);
 
-#ifdef HAVE_MPI
-  // MPI is allowed wait until MPI_Init to fill in argc and argv,
-  // so we may have to call MPI_Init before we even know that we
-  // want an MPIMessageGrp.  The command name is used to let mpqc
-  // know that an early init is needed.
-  if (!strcmp(ExEnv::program_name(), "mpqc-mpi")) {
-    MPI_Init(&argc, &argv);
-  }
+  Ref<MessageGrp> grp;
+#if defined(HAVE_MPI) && defined(ALWAYS_USE_MPI)
+  grp = new MPIMessageGrp(&argc, &argv);
 #endif
 
   // parse commandline options
@@ -233,7 +228,9 @@ main(int argc, char *argv[])
   options.enroll("threadgrp", GetLongOpt::MandatoryValue,
                  "which thread group to use", 0);
   options.enroll("memorygrp", GetLongOpt::MandatoryValue,
-                 "which thread group to use", 0);
+                 "which memory group to use", 0);
+  options.enroll("integral", GetLongOpt::MandatoryValue,
+                 "which integral evaluator to use", 0);
   options.enroll("l", GetLongOpt::MandatoryValue, "basis set limit", "0");
   options.enroll("W", GetLongOpt::MandatoryValue,
                  "set the working directory", ".");
@@ -256,16 +253,16 @@ main(int argc, char *argv[])
   }
 
   if (options.retrieve("h")) {
-    ExEnv::out() << node0
+    ExEnv::out0()
          << indent << "MPQC version " << SC_VERSION << endl
          << indent << "compiled for " << TARGET_ARCH << endl
          << SCFormIO::copyright << endl;
-    options.usage(ExEnv::out());
+    options.usage(ExEnv::out0());
     exit(0);
   }
   
   if (options.retrieve("v")) {
-    ExEnv::out() << node0
+    ExEnv::out0()
          << indent << "MPQC version " << SC_VERSION << endl
          << indent << "compiled for " << TARGET_ARCH << endl
          << SCFormIO::copyright;
@@ -273,7 +270,7 @@ main(int argc, char *argv[])
   }
   
   if (options.retrieve("w")) {
-    ExEnv::out() << node0
+    ExEnv::out0()
          << indent << "MPQC version " << SC_VERSION << endl
          << indent << "compiled for " << TARGET_ARCH << endl
          << SCFormIO::copyright << endl
@@ -282,7 +279,7 @@ main(int argc, char *argv[])
   }
   
   if (options.retrieve("L")) {
-    ExEnv::out() << node0
+    ExEnv::out0()
          << indent << "MPQC version " << SC_VERSION << endl
          << indent << "compiled for " << TARGET_ARCH << endl
          << SCFormIO::copyright << endl
@@ -304,15 +301,12 @@ main(int argc, char *argv[])
     generic_input = argv[optind];
   }
   else {
-    ExEnv::out() << ExEnv::program_name()
-                 << ": extra arguments given"
-                 << endl;
     options.usage();
-    abort();
+    throw invalid_argument("extra arguments given");
   }
 
   // get the message group.  first try the commandline and environment
-  Ref<MessageGrp> grp = MessageGrp::initial_messagegrp(argc, argv);
+  if (grp.null()) grp = MessageGrp::initial_messagegrp(argc, argv);
   if (grp.nonnull())
     MessageGrp::set_default_messagegrp(grp);
   else
@@ -322,9 +316,8 @@ main(int argc, char *argv[])
     generic_input = "mpqc.in";
     }
   else if (object_input && generic_input) {
-    ExEnv::out() << ExEnv::program_name()
-                 << ": only one of -f and a file argument can be given" <<endl;
-    abort();
+    options.usage();
+    throw invalid_argument("only one of -f and a file argument can be given");
     }
 
   const char *input;
@@ -372,7 +365,7 @@ main(int argc, char *argv[])
     MPQCIn mpqcin;
     char *simple_input_text = mpqcin.parse_string(in_char_array);
     if (options.retrieve("i")) {
-      ExEnv::out() << "Generated object-oriented input file:" << endl
+      ExEnv::out0() << "Generated object-oriented input file:" << endl
                    << simple_input_text
                    << endl;
       exit(0);
@@ -388,7 +381,7 @@ main(int argc, char *argv[])
   delete[] in_char_array;
 
   if (options.retrieve("k")) parsedkv->verbose(1);
-  Ref<KeyVal> keyval = new PrefixKeyVal("mpqc",parsedkv.pointer());
+  Ref<KeyVal> keyval = new PrefixKeyVal(parsedkv.pointer(),"mpqc");
 
   // get the basename for output files
   int nfilebase = (int) (::strrchr(input, '.') - input);
@@ -398,8 +391,8 @@ main(int argc, char *argv[])
   SCFormIO::set_default_basename(basename);
 
   // set up output classes
-  SCFormIO::setindent(ExEnv::out(), 2);
-  SCFormIO::setindent(ExEnv::err(), 2);
+  SCFormIO::setindent(ExEnv::outn(), 2);
+  SCFormIO::setindent(ExEnv::errn(), 2);
   SCFormIO::setindent(cout, 2);
   SCFormIO::setindent(cerr, 2);
 
@@ -424,13 +417,13 @@ main(int argc, char *argv[])
   int ntitle1 = sizeof(title1);
   const char title2[] = "Version " SC_VERSION;
   int ntitle2 = sizeof(title2);
-  ExEnv::out() << node0 << endl;
-  ExEnv::out() << node0 << indent;
-  for (i=0; i<(80-ntitle1)/2; i++) ExEnv::out() << node0 << ' ';
-  ExEnv::out() << node0 << title1 << endl;
-  ExEnv::out() << node0 << indent;
-  for (i=0; i<(80-ntitle2)/2; i++) ExEnv::out() << node0 << ' ';
-  ExEnv::out() << node0 << title2 << endl << endl;
+  ExEnv::out0() << endl;
+  ExEnv::out0() << indent;
+  for (i=0; i<(80-ntitle1)/2; i++) ExEnv::out0() << ' ';
+  ExEnv::out0() << title1 << endl;
+  ExEnv::out0() << indent;
+  for (i=0; i<(80-ntitle2)/2; i++) ExEnv::out0() << ' ';
+  ExEnv::out0() << title2 << endl << endl;
 
   const char *tstr = 0;
 #if defined(HAVE_TIME) && defined(HAVE_CTIME)
@@ -442,7 +435,7 @@ main(int argc, char *argv[])
     tstr = "UNKNOWN";
   }
 
-  ExEnv::out() << node0
+  ExEnv::out0()
        << indent << scprintf("Machine:    %s", TARGET_ARCH) << endl
        << indent << scprintf("User:       %s@%s",
                              ExEnv::username(), ExEnv::hostname()) << endl
@@ -476,7 +469,7 @@ main(int argc, char *argv[])
   else
     memory = MemoryGrp::get_default_memorygrp();
 
-  ExEnv::out() << node0 << indent
+  ExEnv::out0() << indent
        << "Using " << grp->class_name()
        << " for message passing (number of nodes = " << grp->n() << ")." << endl
        << indent
@@ -504,6 +497,16 @@ main(int argc, char *argv[])
       dynamic_cast<SCMatrixKit*>(
         keyval->describedclassvalue("matrixkit").pointer()));
 
+  // get the integral factory. first try commandline and environment
+  Ref<Integral> integral = Integral::initial_integral(argc, argv);
+  if (integral.nonnull())
+    Integral::set_default_integral(integral);
+  else
+    integral = Integral::get_default_integral();
+  ExEnv::out0() << endl << indent
+       << "Using " << integral->class_name()
+       << " by default for molecular integrals evaluation" << endl << endl;
+  
   // check for a molecular energy and optimizer
   KeyValValueString molnamedef(basename);
   char * molname = keyval->pcharvalue("filename", molnamedef);
@@ -517,11 +520,18 @@ main(int argc, char *argv[])
   char * restartfile = keyval->pcharvalue("restart_file", restartfiledef);
   
   char * wfn_file = keyval->pcharvalue("wfn_file");
-  
+  if (wfn_file == 0) {
+    wfn_file = new char[strlen(molname)+6];
+    sprintf(wfn_file,"%s.wfn",molname);
+  }
+  char *mole_ckpt_file = new char[strlen(wfn_file)+1];
+  sprintf(mole_ckpt_file,"%s",wfn_file);
+
   int restart = keyval->booleanvalue("restart",truevalue);
 
   int checkpoint = keyval->booleanvalue("checkpoint",truevalue);
-
+  int checkpoint_freq = keyval->intvalue("checkpoint_freq",KeyValValueint(1));
+  
   int savestate = keyval->booleanvalue("savestate",truevalue);
 
   struct stat sb;
@@ -545,7 +555,7 @@ main(int argc, char *argv[])
     char *suf = strrchr(restartfile,'.');
     if (!strcmp(suf,".wfn")) {
       mole << SavableState::key_restore_state(si,"mole");
-      ExEnv::out() << node0 << endl
+      ExEnv::out0() << endl
                    << indent << "Restored <" << mole->class_name()
                    << "> from " << restartfile << endl;
 
@@ -557,7 +567,7 @@ main(int argc, char *argv[])
       opt << SavableState::key_restore_state(si,"opt");
       if (opt.nonnull()) {
         mole << opt->function();
-        ExEnv::out() << node0 << endl << indent
+        ExEnv::out0() << endl << indent
              << "Restored <Optimize> from " << restartfile << endl;
       }
     }
@@ -568,8 +578,14 @@ main(int argc, char *argv[])
 
   if (mole.nonnull()) {
     MolecularFormula mf(mole->molecule());
-    ExEnv::out() << node0 << endl << indent
+    ExEnv::out0() << endl << indent
          << "Molecular formula " << mf.formula() << endl;
+    if (checkpoint) {
+      mole->set_checkpoint();
+      if (grp->me() == 0) mole->set_checkpoint_file(mole_ckpt_file);
+      else mole->set_checkpoint_file(devnull);
+      mole->set_checkpoint_freq(checkpoint_freq);
+    }
   }
 
   if (checkpoint && opt.nonnull()) {
@@ -590,7 +606,7 @@ main(int argc, char *argv[])
   if (limit) {
     Ref<Wavefunction> wfn; wfn << mole;
     if (wfn.nonnull() && wfn->ao_dimension()->n() > limit) {
-      ExEnv::out() << node0 << endl << indent
+      ExEnv::out0() << endl << indent
            << "The limit of " << limit << " basis functions has been exceeded."
            << endl;
       check = 1;
@@ -598,7 +614,7 @@ main(int argc, char *argv[])
   }
 
   if (check) {
-    ExEnv::out() << node0 << endl << indent
+    ExEnv::out0() << endl << indent
          << "Exiting since the check option is on." << endl;
     exit(0);
   }
@@ -620,7 +636,7 @@ main(int argc, char *argv[])
   // sanity checks for the benefit of reasonable looking output
   if (opt.null()) do_opt=0;
   
-  ExEnv::out() << node0 << endl << indent
+  ExEnv::out0() << endl << indent
        << "MPQC options:" << endl << incindent
        << indent << "matrixkit     = <"
        << SCMatrixKit::default_matrixkit()->class_name() << ">" << endl
@@ -644,7 +660,7 @@ main(int argc, char *argv[])
   if (mole.nonnull()) {
     if (((do_opt && opt.nonnull()) || do_grad)
         && !mole->gradient_implemented()) {
-      ExEnv::out() << node0 << indent
+      ExEnv::out0() << indent
            << "WARNING: optimization or gradient requested but the given"
            << endl
            << "         MolecularEnergy object cannot do gradients."
@@ -654,29 +670,29 @@ main(int argc, char *argv[])
     if (do_opt && opt.nonnull() && mole->gradient_implemented()) {
       int result = opt->optimize();
       if (result) {
-        ExEnv::out() << node0 << indent
+        ExEnv::out0() << indent
              << "The optimization has converged." << endl << endl;
-        ExEnv::out() << node0 << indent
+        ExEnv::out0() << indent
              << scprintf("Value of the MolecularEnergy: %15.10f",
                          mole->energy())
              << endl << endl;
       } else {
-        ExEnv::out() << node0 << indent
+        ExEnv::out0() << indent
              << "The optimization has NOT converged." << endl << endl;
         ready_for_freq = 0;
       }
     } else if (do_grad && mole->gradient_implemented()) {
       mole->do_gradient(1);
-      ExEnv::out() << node0 << endl << indent
+      ExEnv::out0() << endl << indent
            << scprintf("Value of the MolecularEnergy: %15.10f",
                        mole->energy())
            << endl;
       if (mole->value_result().actual_accuracy()
           > mole->value_result().desired_accuracy()) {
-        ExEnv::out() << node0 << indent
+        ExEnv::out0() << indent
              << "WARNING: desired accuracy not achieved in energy" << endl;
       }
-      ExEnv::out() << node0 << endl;
+      ExEnv::out0() << endl;
       // Use result_noupdate since the energy might not have converged
       // to the desired accuracy in which case grabbing the result will
       // start up the calculation again.  However the gradient might
@@ -693,12 +709,12 @@ main(int argc, char *argv[])
         grad.print("Gradient of the MolecularEnergy:");
         if (mole->gradient_result().actual_accuracy()
             > mole->gradient_result().desired_accuracy()) {
-          ExEnv::out() << node0 << indent
+          ExEnv::out0() << indent
                << "WARNING: desired accuracy not achieved in gradient" << endl;
         }
       }
     } else if (do_energy && mole->value_implemented()) {
-      ExEnv::out() << node0 << endl << indent
+      ExEnv::out0() << endl << indent
            << scprintf("Value of the MolecularEnergy: %15.10f",
                        mole->energy())
            << endl << endl;
@@ -766,7 +782,7 @@ main(int argc, char *argv[])
       xhessian = molhess->cartesian_hessian();
     }
     else {
-      ExEnv::out() << "mpqc: WARNING: Frequencies cannot be computed" << endl;
+      ExEnv::out0() << "mpqc: WARNING: Frequencies cannot be computed" << endl;
     }
 
     if (xhessian.nonnull()) {
@@ -836,7 +852,7 @@ main(int argc, char *argv[])
 
   if (mole.nonnull()) {
     if (print_mole)
-      mole->print(ExEnv::out());
+      mole->print(ExEnv::out0());
 
     if (do_pdb && grp->me() == 0) {
       ckptfile = new char[strlen(molname)+5];
@@ -848,23 +864,23 @@ main(int argc, char *argv[])
     
   }
   else {
-    ExEnv::out() << node0 << "mpqc: The molecular energy object is null" << endl
+    ExEnv::out0() << "mpqc: The molecular energy object is null" << endl
          << " make sure \"mole\" specifies a MolecularEnergy derivative"
          << endl;
   }
 
   if (parsedkv->have_unseen()) {
-    ExEnv::out() << node0 << indent
+    ExEnv::out0() << indent
          << "The following keywords in \"" << input << "\" were ignored:"
          << endl;
-    ExEnv::out() << incindent;
-    parsedkv->print_unseen(ExEnv::out()<<node0);
-    ExEnv::out() << decindent;
-    ExEnv::out() << node0 << endl;
+    ExEnv::out0() << incindent;
+    parsedkv->print_unseen(ExEnv::out0());
+    ExEnv::out0() << decindent;
+    ExEnv::out0() << endl;
   }
 
   if (print_timings)
-    if (tim.nonnull()) tim->print(ExEnv::out());
+    if (tim.nonnull()) tim->print(ExEnv::out0());
 
   delete[] basename;
   delete[] molname;
@@ -878,6 +894,7 @@ main(int argc, char *argv[])
   molhess = 0;
   opt = 0;
   mole = 0;
+  integral = 0;
   debugger = 0;
   thread = 0;
   tim = 0;
@@ -894,17 +911,45 @@ main(int argc, char *argv[])
   if (!tstr) {
     tstr = "UNKNOWN";
   }
-  ExEnv::out() << node0 << endl
+  ExEnv::out0() << endl
                << indent << scprintf("End Time: %s", tstr) << endl;
 
   if (output != 0) {
-    ExEnv::set_out(&ExEnv::out());
+    ExEnv::set_out(&cout);
     delete outstream;
   }
 
   return 0;
 }
 
+int
+main(int argc, char *argv[])
+{
+  try {
+    try_main(argc, argv);
+  }
+  catch (bad_alloc &e) {
+    cout << argv[0] << ": ERROR: MEMORY ALLOCATION FAILED:" << endl
+         << e.what()
+         << endl;
+    clean_up();
+    throw;
+  }
+  catch (exception &e) {
+    cout << argv[0] << ": ERROR: EXCEPTION RAISED:" << endl
+         << e.what()
+         << endl;
+    clean_up();
+    throw;
+  }
+  catch (...) {
+    cout << argv[0] << ": ERROR: UNKNOWN EXCEPTION RAISED" << endl;
+    clean_up();
+    throw;
+  }
+  return 0;
+}
+
 /////////////////////////////////////////////////////////////////////////////
 
 // Local Variables:
